@@ -2,12 +2,11 @@
 
 # IMPORTANT: evaluation has to be imported first since energyflow must be imported before torch
 # See https://github.com/pkomiske/EnergyFlow/issues/24
-import jetnet.evaluation
-import jetnet.datasets
-import jetnet.losses
-import jetnet.utils
-
-import jetnet.datasets.utils
+import jetnet.datasets  # noqa: F401
 import jetnet.datasets.normalisations
+import jetnet.datasets.utils
+import jetnet.evaluation
+import jetnet.losses
+import jetnet.utils  # noqa: F401
 
-__version__ = "0.2.2"
+__version__ = "0.2.3"
