@@ -1,18 +1,16 @@
-from typing import Callable, List, Set, Union, Optional, Tuple
+from typing import Callable, List, Optional, Set, Tuple, Union
 
 import numpy as np
 
-import logging
-
 from .dataset import JetDataset
+from .normalisations import NormaliseABC
 from .utils import (
     checkConvertElements,
     checkDownloadZenodoDataset,
-    getOrderedFeatures,
-    checkStrToList,
     checkListNotEmpty,
+    checkStrToList,
+    getOrderedFeatures,
 )
-from .normalisations import NormaliseABC
 
 
 class TopTagging(JetDataset):
@@ -113,9 +111,10 @@ class TopTagging(JetDataset):
         Returns:
             (Tuple[Optional[np.ndarray], Optional[np.ndarray]]): particle data, jet data
         """
-        assert (
-            num_particles <= cls.max_num_particles
-        ), f"num_particles {num_particles} exceeds max number of particles in the dataset {cls.max_num_particles}"
+        assert num_particles <= cls.max_num_particles, (
+            f"num_particles {num_particles} exceeds max number of "
+            + f"particles in the dataset {cls.max_num_particles}"
+        )
 
         jet_type = checkConvertElements(jet_type, cls.jet_types, ntype="jet type")
         type_indices = [cls.jet_types.index(t) for t in jet_type]
@@ -175,7 +174,7 @@ class TopTagging(JetDataset):
         ret = f"Including {self.jet_type} jets"
 
         if self.split == "all":
-            ret += f"\nUsing all data (no split)"
+            ret += "\nUsing all data (no split)"
         else:
             ret += f"\nSplit into {self.split} data out of {self.splits} possible splits"
 
